@@ -1,12 +1,11 @@
 #!/usr/bin/perl
 ###########################################################################
-# Makefile for ABI Compliance Checker
+# A makefile to install the tool
 # Install/remove the tool for GNU/Linux, FreeBSD and Mac OS X
 #
 # Copyright (C) 2009-2011 Institute for System Programming, RAS
 # Copyright (C) 2011-2012 Nokia Corporation and/or its subsidiary(-ies)
-# Copyright (C) 2011-2012 ROSA Laboratory
-# Copyright (C) 2012-2015 Andrey Ponomarenko's ABI Laboratory
+# Copyright (C) 2012-2016 Andrey Ponomarenko's ABI Laboratory
 #
 # Written by Andrey Ponomarenko
 #
@@ -23,6 +22,7 @@
 # and the GNU Lesser General Public License along with this program.
 # If not, see <http://www.gnu.org/licenses/>.
 ###########################################################################
+use strict;
 use Getopt::Long;
 Getopt::Long::Configure ("posix_default", "no_ignore_case");
 use File::Path qw(mkpath rmtree);
@@ -32,7 +32,6 @@ use File::Basename qw(dirname);
 use Cwd qw(abs_path);
 use File::Find;
 use Config;
-use strict;
 
 my $TOOL_SNAME = "abi-compliance-checker";
 my $ARCHIVE_DIR = abs_path(dirname($0));
@@ -273,18 +272,19 @@ sub copyDir($$)
 sub readFile($)
 {
     my $Path = $_[0];
-    return "" if(not $Path or not -f $Path);
+    
     open(FILE, $Path) || die ("can't open file \'$Path\': $!\n");
     local $/ = undef;
     my $Content = <FILE>;
     close(FILE);
+    
     return $Content;
 }
 
 sub writeFile($$)
 {
     my ($Path, $Content) = @_;
-    return if(not $Path);
+    
     open(FILE, ">".$Path) || die ("can't open file \'$Path\': $!\n");
     print FILE $Content;
     close(FILE);
